/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __image_ImageRaw_h__
#define __image_ImageRaw_h__

namespace fe
{

/**************************************************************************//**
    @brief Image handling using raw buffers

	@ingroup image

	Currently, no file IO is supported.
	A raw byte dump might be added later.
*//***************************************************************************/
class ImageRaw: virtual public ImageI
{
	public:
				ImageRaw(void);
virtual			~ImageRaw(void);

				//* As ImageI
virtual	U32		createSelect(void);
virtual U32		loadSelect(String filename)				{ return m_current; }
virtual U32		interpretSelect(void* data,U32 size)	{ return m_current; }
virtual U32		interpretSelect(String a_source)		{ return m_current; }
virtual	BWORD	save(String filename)					{ return FALSE; }
virtual void	select(U32 id);
virtual	void	unload(U32 id);

virtual	void	setFormat(ImageI::Format format);
virtual	ImageI::Format	format(void) const
				{	return m_current<m_buffers.size()?
							m_buffers[m_current].m_format: ImageI::e_none; }

virtual	void	resize(U32 width,U32 height,U32 depth);
virtual	void	replaceRegion(U32 x,U32 y,U32 z,
						U32 width,U32 height,U32 depth,void* data);

virtual	U32		width(void) const
				{	return m_current<m_buffers.size()?
							m_buffers[m_current].m_width: 0; }
virtual	U32		height(void) const
				{	return m_current<m_buffers.size()?
							m_buffers[m_current].m_height: 0; }
virtual	U32		depth(void) const
				{	return m_current<m_buffers.size()?
							m_buffers[m_current].m_depth: 0; }
virtual	void*	raw(void) const
				{	return m_current<m_buffers.size()?
							m_buffers[m_current].m_pData: NULL; }

virtual	U32		regionCount(void) const					{ return 0; }
virtual	String	regionName(U32 a_regionIndex) const		{ return ""; }
virtual	Box2i	regionBox(String a_regionName) const
				{	return Box2i(Vector2i(0,0),Vector2i(0,0)); }
virtual	String	pickRegion(I32 a_x,I32 a_y) const		{ return ""; }

	private:
	class Buffer
	{
		public:

					Buffer(void):
						m_created(FALSE),
						m_pData(NULL)
					{ reset(); }
					~Buffer(void)
					{
						if(m_pData)
						{
							deallocate(m_pData);
						}
					}
			void	reset(void)
					{
						if(m_pData)
						{
							deallocate(m_pData);
						}
						m_pData=NULL;
						m_width=0;
						m_height=0;
						m_depth=0;
						m_format=ImageI::e_none;
					}
							
			BWORD			m_created;
			U32				m_width;
			U32				m_height;
			U32				m_depth;
			ImageI::Format	m_format;
			void*			m_pData;
	};

		std::vector<ImageRaw::Buffer>	m_buffers;
		U32								m_current;

static
const	U32	m_bytes[4];
};

} // namespace

#endif /* __image_ImageRaw_h__ */
