import os
import re
import sys
import copy
forge = sys.modules["forge"]

# NOTE: The Maya version of boost may be different.
# The boost-regex appears to be sensitive and they don't provide the dso.
# Moving aside their include/boost seems to fix regex,
# but it leaves their other boost dso's cross-versioned with system includes.

def setup(module):
	module.cppmap = {}
	test_include = {}
	test_cpp = {}
	test_link = {}

	deplibs = forge.corelibs+ [
				"fexDrawDLLib",
				"fexMetaDLLib",
				"fexSurfaceDLLib",
				"fexWindowLib"	]

	fe_maya_oplibs = ''
	if os.path.exists('ext/grass'):
		fe_maya_oplibs += ':fexGrassDL'
		sys.stdout.write("grass ")
	if os.path.exists('ext/vegetation'):
		fe_maya_oplibs += ':fexVegetationDL'
		sys.stdout.write("vegetation ")

	for maya_source in forge.maya_sources:
		if not os.path.exists(maya_source[1]) or not os.path.exists(maya_source[2]):
			sys.stdout.write("-" + maya_source[0] + " ")
			continue

		if maya_source[0] == "":
			sys.stdout.write(". ")
		else:
			sys.stdout.write(maya_source[0] + " ")

		srcList = [	"maya.pmh",
					"MayaBrush",
					"MayaDraw",
					"MayaGraph",
					"MayaNode",
					"SurfaceAccessibleMaya",
					"SurfaceAccessorMaya",
					"SurfaceTrianglesMaya",
					"assertMaya",
					"mayaDL" ]

		variant = maya_source[0]
		if variant != "":
			variantPath = module.modPath + '/' + variant
			if os.path.lexists(variantPath) == 0:
				os.symlink('.', variantPath)

			srcListVariant = []
			for src in srcList:
				srcListVariant += [ variant + '/' + src ]
			srcList = srcListVariant

		variantSplit = maya_source[1].split('/')
		variantSplit = variantSplit[len(variantSplit)-2].split('-')
		maya_major = variantSplit[0].replace('maya', '')

		dllname = "fexMayaDL" + variant

		dll = module.DLL( dllname, srcList )

		fe_maya_libs = dllname + fe_maya_oplibs

		for src in srcList:
			relativePath = forge.RelativePath(forge.rootPath,module.modPath)
			modObjPath = os.path.join(forge.objPath, relativePath)
			objName = modObjPath + '/' + src
			if re.compile(r'(.*)\.pmh$').match(src):
				objName += '.gch'
			else:
				objName += forge.objsuf
			srcTarget = forge.FindTarget(objName)
			srcTarget.includemap = { 'maya_src' : maya_source[1] }
			srcTarget.cppmap = { 'maya_src' :
					' -DFE_MAYA_MAJOR=' + str(maya_major) }
			srcTarget.cppmap['maya_src'] += ' -DFE_MAYA_LIBS=' + fe_maya_libs
			if variant == "":
				test_include = copy.deepcopy(srcTarget.includemap)
				test_cpp = copy.deepcopy(srcTarget.cppmap)

		# this overrides previous -W flags in codegen
		module.cppmap = {}
		module.cppmap['maya'] = ' -Wno-deprecated -Wall -W -Wno-parentheses -Wno-sign-compare -Wno-reorder -Wno-uninitialized -Wunused -Wno-unused-parameter'

		dll.linkmap = {}
		if forge.fe_os == "FE_LINUX":
			dll.linkmap["mayalibs"] = "-shared"
			dll.linkmap["mayalibs"] += " -L" + maya_source[2];
			dll.linkmap["mayalibs"] += " -lOpenMaya -lOpenMayaUI"
			dll.linkmap["mayalibs"] += " -Xlinker -rpath='$ORIGIN/../'"
		elif forge.fe_os == "FE_WIN32" or forge.fe_os == "FE_WIN64":
			dll.linkmap["mayalibs"] = "TODO"

		if variant == "":
			test_link = copy.deepcopy(dll.linkmap)

		forge.deps( [ dllname + "Lib" ], deplibs )

		# make sym links for maya to look at
		# TODO should work on non-linux
		mayaDso = 'lib' + dllname + '.so'

		source = os.path.join('..', mayaDso)

		for suffix in [ '', variant ]:
			mayaPath = os.path.join(forge.libPath, 'maya') + suffix
			if os.path.isdir(mayaPath) == 0:
				os.makedirs(mayaPath, 0755)

			dest = os.path.join(mayaPath, 'libfexMayaDL.so')
			if os.path.lexists(dest) == 0:
				os.symlink(source, dest)

	test = module.Module('test')
	test.includemap = test_include
	test.cppmap = {}
	test.cppmap = copy.deepcopy(module.cppmap)
	if 'maya_src' in test_cpp:
		test.cppmap['maya'] += test_cpp['maya_src']
	test.linkmap = copy.deepcopy(test_link)

def auto(module):
	test_file = """
#define _BOOL
#include <maya/MTypes.h>

int main(void)
{
	return(0);
}
    \n"""

	maya_path = os.getenv('MAYA_ROOT')
	if maya_path:
		forge.includemap['maya'] = os.path.join(maya_path,'include')

	return forge.cctest(test_file)
