/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __operate_OperatorThreaded_h__
#define __operate_OperatorThreaded_h__

namespace fe
{

/**************************************************************************//**
	@brief Handler to move curves away from a collider

	@ingroup operate
*//***************************************************************************/
class FE_DL_EXPORT OperatorThreaded:
	virtual public OperatorSurfaceCommon,
	public Initialize<OperatorThreaded>
{
	public:

					OperatorThreaded(void):
						m_threadsAlive(0),
						m_waitForJobs(FALSE)								{}
virtual				~OperatorThreaded(void);

		void		initialize(void);

		using		OperatorSurfaceCommon::accessOutput;

		BWORD		accessOutput(
							sp<SurfaceAccessibleI>& a_rspAccessible,
							Record& a_rSignal,Message a_message=e_error)
					{
//						adjustThreads();
						return OperatorSurfaceCommon::accessOutput(
								a_rspAccessible,a_rSignal,a_message);
					}

					//* As HandlerI
virtual	void		handle(Record& a_rSignal);

virtual	void		run(I32 a_id,sp<SurfaceRange> a_spRange)				=0;

		void		setFullRange(sp<SurfaceRange> a_spRange);
		void		setNonAtomicRange(I32 a_start,I32 a_end);
		void		setAtomicRange(
							sp<SurfaceAccessibleI> a_spSurfaceAccessibleI,
							OperatorSurfaceCommon::AtomicChange a_atomicChange,
							String a_group="");

	private:
		void		finish(void);
		void		adjustThreads(void);

	class Worker: public Thread::Functor
	{
		public:
						Worker(U32 a_id,
							sp< JobQueue< sp<SurfaceRange> > > a_spJobQueue);
	virtual				~Worker(void);

	virtual	void		operate(void);

			void		setObject(sp<Counted> spObject)
						{	m_spOperatorThreaded=spObject; }

		private:
			U32									m_id;
			sp< JobQueue< sp<SurfaceRange> > >	m_spJobQueue;
			sp<OperatorThreaded>				m_spOperatorThreaded;
	};

		sp<SurfaceRange>						m_spFullRange;

		sp< Gang< Worker,sp<SurfaceRange> > >	m_spGang;
		U32										m_threadsAlive;
		BWORD									m_waitForJobs;
};

} /* namespace */

#endif /* __operate_OperatorThreaded_h__ */

