/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define FE_XRO_DEBUG		FALSE

#include "operator/operator.pmh"

using namespace fe;

void XRayOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<I32>("ImageScale")=2;
	catalog<String>("ImageScale","label")="Image Scale";
	catalog<I32>("ImageScale","min")=1;
	catalog<I32>("ImageScale","high")=8;
	catalog<I32>("ImageScale","max")=32;

	catalog< sp<Component> >("Brush");
	catalog<String>("Brush","implementation")="DrawI";

	catalog< sp<Component> >("Input Surface");

	m_spRayTrace=registry()->create("DrawI.DrawRayTrace");
}

void XRayOp::handle(Record& a_rSignal)
{
	const I32 size=catalog<I32>("ImageScale")*64;

	sp<DrawI> spDrawOverlay;
	accessBrushOverlay(spDrawOverlay,a_rSignal,e_quiet);

	if(spDrawOverlay.isValid())
	{
#if FE_XRO_DEBUG
		feLog("XRayOp::handle brush\n");
#endif

		m_brushed=TRUE;

		m_event.bind(windowEvent(a_rSignal));
		const I32 mouseX=m_event.mouseX();
		const I32 mouseY=m_event.mouseY();

		const Color red(1.0,0.0,0.0,1.0);

		const SpatialVector& rRayOrigin=rayOrigin(a_rSignal);
		const SpatialVector& rRayDirection=rayDirection(a_rSignal);

		//* temporarily alter ViewMode
		sp<ViewI> spView=spDrawOverlay->view();
		sp<ViewMode> spViewMode=spView->viewMode();

		const Box2i origViewport=spViewMode->viewport();

		Real origFovy;
		Real nearPlane;
		Real farPlane;
		spViewMode->getPerspective(origFovy,nearPlane,farPlane);

		SpatialTransform cameraOrig;
		invert(cameraOrig,spViewMode->camera());
		const SpatialVector cameraY=cameraOrig.column(1);

		Box2i viewport;
		set(viewport,0,0,size,size);
		spViewMode->setViewport(viewport);

		SpatialTransform offset;
		setIdentity(offset);
		translate(offset,SpatialVector(mouseX-size/2,mouseY-size/2));

		const Real fovy=origFovy*size/height(origViewport);

		SpatialTransform camera;
		makeFrameCameraZ(camera,rRayOrigin,cameraY,rRayDirection);

		SpatialTransform invCamera;
		invert(invCamera,camera);
		spViewMode->setCamera(invCamera);
		spViewMode->setPerspective(fovy,nearPlane,farPlane);

		m_spRayTrace->setTransform(offset);
		m_spRayTrace->setDrawChain(spDrawOverlay);

		const I32 surfaceCount=m_surfaceArray.size();
		for(I32 surfaceIndex=0;surfaceIndex<surfaceCount;surfaceIndex++)
		{
			m_spRayTrace->draw(m_surfaceArray[surfaceIndex],&red);
		}

		m_spRayTrace->flush();

		//* restore
		spViewMode->setViewport(origViewport);
		spViewMode->setPerspective(origFovy,nearPlane,farPlane);

		return;
	}

	const BWORD paramChanged=!m_brushed;
	m_brushed=FALSE;

#if FE_XRO_DEBUG
	feLog("XRayOp::handle changed %d\n",paramChanged);
#endif

	if(paramChanged)
	{
		m_surfaceArray.clear();

		I32 surfaceIndex=0;
		while(TRUE)
		{
#if FE_XRO_DEBUG
				feLog("XRayOp::handle prepare surface %d\n",surfaceIndex);
#endif

			sp<SurfaceI> spInput;
			if(!access(spInput,"Input Surface",e_quiet,surfaceIndex++))
			{
				break;
			}

			spInput->prepareForSearch();
			m_surfaceArray.push_back(spInput);
		}
	}
}
